
## SFKM algorithm: empty cluster remedy 
fastkmed2 <- function (distdata, ncluster, iterate = 10, init = NULL) 
{
  if (any(is.na(distdata))) 
    stop("Cannot handle missing values!")
  if ((is.matrix(distdata) || class(distdata) == "dist") == 
      FALSE) 
    stop("The distdata must be a matrix or a dist object!")
  if (is.matrix(distdata) == TRUE) {
    nr <- nrow(distdata)
    nc <- ncol(distdata)
    if (nc != nr) 
      stop("The distdata is not an x n distance matrix!")
  }
  if (class(distdata) == "dist") 
    distdata <- as.matrix(distdata)
  if (is.null(rownames(distdata)) == TRUE) 
    rownames(distdata) <- 1:nrow(distdata)
  n <- nrow(distdata)
  index <- 1:n
  if (is.null(init) == TRUE) {
    distsum <- colSums(distdata/sum(distdata))
    names(distsum) <- index
    iduniq <- as.numeric(names(distsum[!duplicated(distsum)]))
    distuniq <- unique(distsum)
    names(distuniq) <- iduniq
    sorted_object <- as.numeric(names(sort(distuniq)))
    medoid_init <- sorted_object[1:ncluster]
  }
  else {
    if (ncluster != length(init)) 
      stop("The initial medoids must equal to the number of clusters")
    medoid_init <- init
  }
  dist_0 <- distdata[, medoid_init]
  
  for (i in 1:ncluster) {
    dist_0[names(dist_0[,i]) == colnames(dist_0)[i], i] <- -1
  }
  
  member_0 <- apply(dist_0, 1, which.min)
  E0 <- sum(apply(dist_0, 1, min))
  row_0 <- row_1 <- medoid_0 <- medoid_1 <- numeric(ncluster)
  for (i in 1:ncluster) {
    row_0[i] <- which.min(rowSums(distdata[member_0 == i, 
                                           member_0 == i, drop = FALSE]))
    medoid_0[i] <- index[member_0 == i][row_0[i]]
  }
  dist_1 <- distdata[, medoid_0]
  
  for (i in 1:ncluster) {
    dist_1[names(dist_1[,i]) == colnames(dist_1)[i], i] <- -1
  }
  
  member_1 <- apply(dist_1, 1, which.min)
  E1 <- sum(apply(dist_1, 1, min))
  for (i in 1:ncluster) {
    row_1[i] <- which.min(rowSums(distdata[member_1 == i, 
                                           member_1 == i, drop = FALSE]))
    medoid_1[i] <- index[member_1 == i][row_1[i]]
  }
  if (sum(sort(medoid_0) != sort(medoid_1)) != 0) {
    x <- 1
    medoid <- vector("list", iterate)
    E <- numeric(iterate)
    repeat {
      dist_0 <- dist_1
      member_0 <- member_1
      E0 <- E1
      row_0 <- row_1
      medoid_0 <- medoid_1
      dist_1 <- distdata[, medoid_0]
      
      for (i in 1:ncluster) {
        dist_1[names(dist_1[,i]) == colnames(dist_1)[i], i] <- -1
      }
      
      member_1 <- apply(dist_1, 1, which.min)
      for (i in 1:ncluster) {
        row_1[i] <- which.min(rowSums(distdata[member_1 == 
                                                 i, member_1 == i, drop = FALSE]))
        medoid_1[i] <- index[member_1 == i][row_1[i]]
      }
      E1 <- sum(apply(dist_1, 1, min))
      medoid[[x]] <- medoid_1
      E[x] <- E1
      if (x == iterate || sum(sort(medoid_0) != sort(medoid_1)) == 
          0) {
        break
      }
      x <- x + 1
    }
    medoid <- do.call(rbind, medoid)
    E <- E[1:x]
    index_E <- which.min(E)
    medoid_2 <- medoid[index_E, ]
    dist_2 <- distdata[, medoid_2]
    
    for (i in 1:ncluster) {
      dist_2[names(dist_2[,i]) == colnames(dist_2)[i], i] <- -1
    }
    
    member_2 <- apply(dist_2, 1, which.min)
    E2 <- sum(apply(dist_2, 1, min))
    names(member_2) <- rownames(distdata)
    result <- list(cluster = member_2, medoid = medoid_2, 
                   minimum_distance = E2)
  }
  else {
    dist_2 <- distdata[, medoid_1]
    
    for (i in 1:ncluster) {
      dist_2[names(dist_2[,i]) == colnames(dist_2)[i], i] <- -1
    }
    
    member_1 <- apply(dist_2, 1, which.min)
    E2 <- sum(apply(dist_2, 1, min))
    result <- list(cluster = member_1, medoid = medoid_1, 
                   minimum_distance = E2)
  }
  return(result)
}

## SKM algorithm
skmedoidR <- function(distdata, ncluster, seeding = 20, iterate = 10){
  
  n <- nrow(distdata)
  
  colnames(distdata) <- rownames(distdata) <- 1:n
  order_object <- sort(colSums(distdata/sum(distdata)))
  sorted_object <- as.numeric(names(order_object)[!duplicated(order_object)])
  
  init1 <- sorted_object[1]
  noninit1 <- sorted_object[-1]
  z  <- vector("list", seeding)
  for (i in 1:seeding) {
    medinit <- c(init1, sample(noninit1, ncluster-1))
    z[[i]] <- fastkmed2(distdata, ncluster, iterate, init = medinit)
  }
  
  min_dist <- numeric(seeding)
  for (i in 1:seeding) {
    min_dist[i] <- z[[i]]$minimum_distance
  }
  result <- z[[which.min(min_dist)]]
  return(result)
}

## all distance function
alldist <- function(data, ncluster, membership, numvar, binvar, catvar) {
  distall <- vector("list", 7) 
  names(distall) <- c("gower", "wishart", "podani", "huang", "harikumar",
                      "Esimma", "Marweco")
  for (i in 1:5) {
    distall[[i]] <- distmix(data, method = names(distall)[i], numvar, binvar, catvar)
  }
  distall[[6]] <- as.matrix(dist(data[,numvar, drop = FALSE])) + 
    matching(data[,binvar , drop = FALSE], data[,binvar, drop = FALSE]) +
    matching(data[,catvar, drop = FALSE], data[,catvar, drop = FALSE])
  distall[[7]] <- distNumeric(as.matrix(data[,numvar, drop = FALSE]), 
                              as.matrix(data[,numvar, drop = FALSE])) + 
    cooccur(data[,catvar, drop = FALSE]) 
  res <- matrix(0, 3, 7, dimnames = list(c("PAM", "FSKM", "KM"),names(distall)))
  for (i in 1:7) {
    pam <- cluster::pam(as.dist(distall[[i]]), ncluster, diss = TRUE)
    ripam <- flexclust::comPart(membership,pam$clustering, "RI")
    fskm <- fastkmed(distall[[i]], ncluster, iterate = 20)
    rifskm <- flexclust::comPart(membership,fskm$cluster, "RI")
    skm <- skmedoidR(distall[[i]], ncluster)
    riskm <- flexclust::comPart(membership,skm$cluster, "RI")
    ri <- rbind(ripam, rifskm, riskm)
    res[,i] <- ri
  }
  return(res)
}

## subsetting function
subsetting <- function(data, ncluster, membership, numvar, binvar, catvar, repli) {
  n <- nrow(data)
  result <- array(0, dim = c(3,7,repli), 
                  dimnames = list(c("PAM", "SFKM", "SKM"),
                                  c("gower", "wishart", "podani", "huang", "harikumar",
                                    "Esimma", "Marweco"),
                                  c(1:repli)))
  for (i in 1:repli) {
    idx <- sample(1:n, n/2)
    dat1 <- data[idx,]
    member1 <- membership[idx]
    result[,,i] <- alldist(dat1, ncluster, member1, numvar, binvar, catvar)
  }
  return(result)
}

## categorized numerical variable function
intv <- function(vec, class) {
  nbase <- (1:(class-1))/class
  nq <- numeric(length(nbase))
  for (i in 1:length(nq)) {
    nq[i] <- quantile(vec, nbase[i])
  }
  res <- c(min(vec), nq, max(vec)) 
  res[1] <- res[1]-1
  return(res)
}

## function to maximaze diagonal
maxdiagonal <- function(mat) {
  tab <- table(mat[,1],mat[,2])
  sorted <- clue::solve_LSAP(tab, maximum = TRUE) #choose label most freq
  idxsort <- as.numeric(sorted)
  names(idxsort) <- 1:ncol(tab)
  idxsort <- as.numeric(names(sort(idxsort)))
  labsingle <- as.factor(idxsort) #a single vector relabel (k number)
  vecno <- mat[mat[,2]!=0,2]
  vecno <- as.factor(vecno)
  levels(vecno) <- labsingle  #relabel the old vector (k number)
  newlab <- as.numeric(as.character(vecno)) #relabel all input
  mat[mat[,2]!=0,2] <- newlab  #put new label in the second column
  return(mat)
}

## function to calculate numerical distance
numdata <- function(data, nclust, idclass, seed = 20) {
  data <- as.matrix(data)
  distgower <- distNumeric(data, data, method = "mrw")
  distwishart <- distNumeric(data, data, method = "sev")
  distpodani <- distNumeric(data, data, method = "ser.2")
  disthuang <- distNumeric(data, data, method = "se")
  disthari <- as.matrix(dist(data, method = "manhattan"))
  distesimma <- as.matrix(dist(data, method = "euclidean"))
  alldist <- list(distgower, distwishart, distpodani, disthuang, disthari, distesimma)
  result <- vector("list", 6)
  for (i in 1:6) {
    skm <- skmedoidR(alldist[[i]], nclust, seeding = seed)$cluster
    classmember <- cbind(skm, idclass)
    maxmembership <- maxdiagonal(classmember)
    result[[i]] <- table(maxmembership[,1], maxmembership[,2])
  }
  accuracy <- sapply(result, function(x) sum(diag(x))/sum(x))
  names(accuracy) <- c("Gower", "Wishart", "Podani", "Huang", "Harikumar", "Esimma")
  return(accuracy)
}

## function to calculate categorical distance
catdata <- function(data, nclust, idclass, seed = 20) {
  for (i in 1:ncol(data)){
    data[,i] <- as.factor(data[,i])
  }
  distgowesim <- matching(data,data)
  distwishart <- matching(data,data)^(1/2)
  distpodani <- (matching(data,data)*ncol(data))^(1/2)
  disthuang <- matching(data,data)*ncol(data)
  distharmar <- cooccur(data)
  alldist <- list(distgowesim, distwishart, distpodani, disthuang, distharmar)
  result <- vector("list", 5)
  for (i in 1:5) {
    skm <- skmedoidR(alldist[[i]], nclust, seeding = seed)$cluster
    classmember <- cbind(skm, idclass)
    maxmembership <- maxdiagonal(classmember)
    result[[i]] <- table(maxmembership[,1], maxmembership[,2])
  }
  accuracy <- sapply(result, function(x) sum(diag(x))/sum(x))
  names(accuracy) <- c("GowerEsimma", "Wishart","Podani", "Huang", "HarikumarMarweco")
  return(accuracy)
}



