library(kmed)
source("00function.R")
set.seed(2019)

nclust <- 4
sep <- 0.2
nsimul <- 50
nvar <- 12
n <- 200

## generate data sets
randclust <- clusterGeneration::genRandomClust(nclust, sepVal = sep, numNonNoisy = nvar, 
                                               numReplicate = 1, clustszind = 3, 
                                               clustSizes = as.numeric(table(sample(1:nclust, n, replace = TRUE))),
                                               outputDatFlag=FALSE, 
                                               outputLogFlag=FALSE, 
                                               outputEmpirical=FALSE, 
                                               outputInfo=FALSE)
randclust2 <- clusterGeneration::genRandomClust(nclust, sepVal = sep, numNonNoisy = nvar, 
                                               numReplicate = 1, clustszind = 3, 
                                               clustSizes = as.numeric(table(sample(1:nclust, n, replace = TRUE))),
                                               outputDatFlag=FALSE, 
                                               outputLogFlag=FALSE, 
                                               outputEmpirical=FALSE, 
                                               outputInfo=FALSE)
randclust3 <- clusterGeneration::genRandomClust(nclust, sepVal = sep, numNonNoisy = nvar, 
                                               numReplicate = 1, clustszind = 3, 
                                               clustSizes = as.numeric(table(sample(1:nclust, n, replace = TRUE))),
                                               outputDatFlag=FALSE, 
                                               outputLogFlag=FALSE, 
                                               outputEmpirical=FALSE, 
                                               outputInfo=FALSE)
randclust4 <- clusterGeneration::genRandomClust(nclust, sepVal = sep, numNonNoisy = nvar, 
                                               numReplicate = 1, clustszind = 3, 
                                               clustSizes = as.numeric(table(sample(1:nclust, n, replace = TRUE))),
                                               outputDatFlag=FALSE, 
                                               outputLogFlag=FALSE, 
                                               outputEmpirical=FALSE, 
                                               outputInfo=FALSE)
dat1 <- randclust$datList$test_1
dat1 <- data.frame(dat1)
classdat1 <- randclust$memList$test_1
for (i in 9:10) {
  dat1[,i] <- dat1[,i] < median(dat1[,i])
}
dat1$x11 <- as.factor(cut(dat1$x11, intv(dat1$x11, 3), labels = (1:3)))
dat1$x12 <- as.factor(cut(dat1$x12, intv(dat1$x12, 5), labels = (1:5)))
dat2 <- randclust2$datList$test_1
dat2 <- data.frame(dat2)
classdat2 <- randclust2$memList$test_1
for (i in 3:10) {
  dat2[,i] <- dat2[,i] < median(dat2[,i])
}
dat2$x11 <- as.factor(cut(dat2$x11, intv(dat2$x11, 3), labels = (1:3)))
dat2$x12 <- as.factor(cut(dat2$x12, intv(dat2$x12, 5), labels = (1:5)))
dat3 <- randclust3$datList$test_1
dat3 <- data.frame(dat3)
classdat3 <- randclust3$memList$test_1
for (i in 3:4) {
  dat3[,i] <- dat3[,i] < median(dat3[,i])
}
for (i in 5:12) {
  dat3[,i] <- as.factor(cut(dat3[,i], intv(dat3[,i], 3), labels = (1:3)))
}
dat4 <- randclust4$datList$test_1
dat4 <- data.frame(dat4)
classdat4 <- randclust4$memList$test_1
for (i in 3:4) {
  dat4[,i] <- dat4[,i] < median(dat4[,i])
}
for (i in 5:12) {
  dat4[,i] <- as.factor(cut(dat4[,i], intv(dat4[,i], 5), labels = (1:5)))
}

## defining: data sets, membership references, variable proportion before simulation 
varsummary <- propidx <- vector("list", 4)
dataset <- list(dat1, dat2, dat3, dat4)
memberset <- list(classdat1, classdat2, classdat3, classdat4)
propidx[[1]] <- list(1:8, 9:10, 11:12) 
propidx[[2]] <- list(1:2, 3:10, 11:12)
propidx[[3]] <- list(1:2, 3:4, 5:12)
propidx[[4]] <- list(1:2, 3:4, 5:12)

## Run the simulation for table 2 in domination part
system.time(
for (i in 1:4) {
  varsummary[[i]] <- subsetting(dataset[[i]], nclust, memberset[[i]], 
                                propidx[[i]][[1]], propidx[[i]][[2]], propidx[[i]][[3]], nsimul)
}
) #+- 298 seconds

## template for the result
names(varsummary) <- paste("dataset", 1:nclust, sep="")
sumdataset <- array(0, dim = c(3,7,4), 
                    dimnames = list(rownames(varsummary$dataset1[,,1]),
                                    colnames(varsummary$dataset1[,,1]),
                                    paste("dataset", 1:4, sep = "")))
pvaldataset <- array(1, dim = c(3,7,4), 
                     dimnames = list(c("PAM-SFKM", "SFKM-SKM", "PAM-SKM"),
                                     colnames(varsummary$dataset1[,,1]),
                                     paste("dataset", 1:4, sep = "")))

## Table 4
for (j in 1:4) {
  for (i in 1:3) {
    sumdataset[i,,j] <- round(apply(varsummary[[j]][i,,], 1, mean),2)
  }
}
for (j in 1:4) {
  for (i in 1:7) {
    ttest12 <- t.test(varsummary[[j]][1,i,], varsummary[[j]][2,i,])
    ttest23 <- t.test(varsummary[[j]][2,i,], varsummary[[j]][3,i,])
    if (sumdataset[1,i,j] == sumdataset[3,i,j]) 
      { 
        pvaldataset[3,i,j] <- NA
    } else  {
      ttest13 <- t.test(varsummary[[j]][1,i,], varsummary[[j]][3,i,])
      pvaldataset[3,i,j] <- round(ttest13$p.value,2)
    }
    pvaldataset[1,i,j] <- round(ttest12$p.value,2)
    pvaldataset[2,i,j] <- round(ttest23$p.value,2)
  }
}
for (i in 1:4) {
  print(xtable::xtable(sumdataset[,,i]))
}


