## SKM function in Rcpp function
system.time(
Rcpp::sourceCpp("path/skmedoid.cpp")
) #+- 21

## generate data sets
library(kmed)
source("00function.R")
set.seed(2018)
nclust <- c(3, 10)
nvar <- 8
n <- c(200, 1000, 3000)
dataset <- vector("list", 2)
names(dataset) <- nclust
for (i in 1:2) {
  a <- vector("list", 3)
  names(a) <- n
  dataset[[i]] <- a
}
system.time(
for (i in 1:2) {
  for (j in 1:3) {
    randclust <- clusterGeneration::genRandomClust(nclust[i], 
                  sepVal = 0.5, numNonNoisy = nvar, 
                  numReplicate = 1, clustszind = 3, 
                  clustSizes = as.numeric(table(sample(1:nclust[i], n[j], replace = TRUE))),
                  outputDatFlag=FALSE, 
                  outputLogFlag=FALSE, 
                  outputEmpirical=FALSE, 
                  outputInfo=FALSE)
    dat1 <- randclust$datList$test_1
    dat1 <- data.frame(dat1)
    for (ii in 5:6) {
      dat1[,ii] <- dat1[,ii] < median(dat1[,ii])
    }
    for (ii in 7:8) {
      dat1[,ii] <- as.factor(cut(dat1[,ii], intv(dat1[,ii], 3), labels = (1:3)))
    }
    dataset[[i]][[j]] <- dat1
  }
}
) #+- 20

distdata <- dataset
system.time(
for (i in 1:2) {
  for (j in 1:3) {
    distdata[[i]][[j]] <- distmix(dataset[[i]][[j]], method = "gower",1:4, 5:6, 7:8)
  }
}
) #+- 14


## run benchmarking
resti <- dataset
repli <- 50
system.time(
for (i in 1:2) {
  for (j in 1:3) {
    n <- nrow(distdata[[i]][[j]])
    ncluster <- nclust[i]
    resti[[i]][[j]] <- matrix(0, repli, 2, dimnames = list(1:repli, c("SKM20", "PAM")))
    for (kk in 1:repli) {
      idx <- sample(1:n, n/2)
      dat1 <- distdata[[i]][[j]][idx,idx]

      ti <- microbenchmark::microbenchmark(skm20 = skmedoidcpp(dat1, ncluster, seeding = 20),
                                           pam = cluster::pam(as.dist(dat1), ncluster, diss = TRUE),
                                           unit = "us", times = 1)
      exti <- ti$time
      names(exti) <- ti$expr
      resti[[i]][[j]][kk,1] <- exti[names(exti) == "skm20"]
      resti[[i]][[j]][kk,2] <- exti[names(exti) == "pam"]
    }
  }
}
) #+- 346

## summary the result
n3 <- n10 <- matrix(0, 3, 2, dimnames = list(c(200, 1000, 3000), c("SKM20", "PAM")))
for (i in 1:3) {
  n3[i,] <- apply(resti$`3`[[i]], 2, mean)
  n10[i,] <- apply(resti$`10`[[i]], 2, mean)
}
dftime2 <- data.frame(mt3 = c(n3)/1e9, mt10 = c(n10)/1e9,
                      Algorithms = rep(c("SKM", "PAM"), each = 3),
                      n = rep(c(200, 1000, 3000), 2)
)

library(ggplot2)

## Figure 2a
p3 <- ggplot(dftime2, aes(x=n, y=mt3, color = Algorithms)) +
  geom_line(aes(linetype=Algorithms), size = 1) +
  xlab("\n Number of objects") + 
  ylab("Seconds \n ")  +
  scale_y_continuous(breaks = seq(0, 1, 0.2)) +
  theme(axis.title = element_text(size=18), 
        plot.caption = element_text(hjust = -0, size = 18, face = "bold"),
        axis.text = element_text(size = 14),
        legend.title=element_text(size=17),
        legend.text=element_text(size=17)) +
  labs(caption = "(a) The number of clusters k = 3") #+ coord_fixed(ratio = 0.8)

## Figure 2b
p10 <- ggplot(dftime2, aes(x=n, y=mt10, color = Algorithms)) +
  geom_line(aes(linetype=Algorithms), size = 1) +
  xlab("\n Number of objects") + 
  ylab("Seconds \n ")  +
  scale_y_continuous(breaks = seq(0, 3.5, 0.5)) +
  theme(axis.title = element_text(size=18), 
        plot.caption = element_text(hjust = -0, size = 18, face = "bold"),
        axis.text = element_text(size = 14),
        legend.title=element_text(size=17),
        legend.text=element_text(size=17)) +
  labs(caption = "(b) The number of clusters k = 10") #+ 

gridExtra::grid.arrange(p3, p10, ncol = 2)

