#include <RcppArmadillo.h>
// [[Rcpp::depends(RcppArmadillo)]]
using namespace Rcpp;

IntegerVector initC(NumericMatrix x, int nclust) {
  
  //**** find initial medoid //
  int n = x.nrow();
  NumericVector rowmat, sumat(n), sumorder;
  double total = 0;
  for (int i = 0; i < n; i ++) {
    rowmat = x(i,_);
    total += sum(rowmat);
  }
  NumericMatrix stdx = x/ total;
  for (int i = 0; i < n; i ++) {
    sumat[i] = sum(stdx(i,_));
  }
  IntegerVector initial;
  sumorder = clone(sumat).sort();
  initial = match(sumorder, sumat);
  initial = initial[seq_len(nclust)-1];
  return (initial);
  //return List::create(initial, sumorder);
}

IntegerVector memberC(NumericMatrix x, IntegerVector y) {
  
  //*** find membership given distance matrix //
  arma::uvec index = as<arma::uvec>(y) - 1;
  arma::mat m = as<arma::mat>(x);
  arma::mat M = m.cols(index);
  NumericMatrix N = Rcpp::wrap(M);
  int n = N.nrow();
  IntegerVector mem(n);
  for (int i = 0; i < n; i++){
    NumericVector cm1 = N(i,_);
    mem[i] = which_min(cm1) + 1;
  }
  return (mem);
}

IntegerVector medoidC(NumericMatrix x, IntegerVector y) {
  
  //*** find medoid each iteration //
  IntegerVector member = sort_unique(y);
  int n = member.size();
  IntegerVector z = seq_len(y.size());
  arma::mat m = as<arma::mat>(x);
  arma::uvec idx = as<arma::uvec>(y) - 1;
  IntegerVector index = y - 1;
  IntegerVector p(n);
  for (int i = 0; i < n; i++) {
    arma::mat M = m.submat(find(idx == i), find(idx == i));
    NumericMatrix N = Rcpp::wrap(M);
    int nsub = N.nrow();
    IntegerVector q = z[index == i];
    NumericVector out(nsub), idclass(nsub);
    for (int j = 0; j < nsub; j++) {
      out[j] = sum(N(j,_));
      idclass[j] = q[j];
    }
    p[i] =  idclass[which_min(out)];
  }
  return (p);
}

double compE(NumericMatrix x, IntegerVector y) {
  
  //*** find membership and loss value given distance matrix //
  arma::uvec index = as<arma::uvec>(y) - 1;
  arma::mat m = as<arma::mat>(x);
  arma::mat M = m.cols(index);
  NumericMatrix N = Rcpp::wrap(M);
  int n = N.nrow();
  NumericVector E(n);
  for (int i = 0; i < n; i++){
    NumericVector cm1 = N(i,_);
    E[i] = min(cm1);
  }
  return (sum(E));
}

// [[Rcpp::export]]
List fastkmedC(NumericMatrix x, int nclust, int iterate = 10) {
  
  //*** cluster result //
  //int n = x.nrow();
  IntegerVector initial = initC(x, nclust); //vector length  = nclust
  IntegerVector member0 = memberC(x, initial), member1, member;//vector length = n
  IntegerVector medoid1 = medoidC(x, member0), medoid2 ,medoid; //vector length = nclust
  
  if (sum(medoid1.sort() != initial.sort())!=0) {
    int i = 0;
    while ( i < iterate) { 
      member1 = memberC(x, medoid1);
      medoid2 = medoidC(x, member1);
      if (sum(medoid1.sort() != medoid2.sort())==0) { break;} //yang diganti
      medoid1 = medoid2;
      i++;
    }
    medoid = medoid2;
    member = memberC(x, medoid2);
  } else {
    medoid = medoid1;
    member = memberC(x, medoid1);
  }
  double E = compE(x, medoid);
  return List::create(Named("cluster") = member, Named("medoid") = medoid,
                      Named("minimum_distance") = E);
}

