#include <RcppArmadillo.h>
// [[Rcpp::depends(RcppArmadillo)]]

using namespace Rcpp;

//' A simple k medoids algorithm using cpp code.
//'
//' @description This cpp algorithm give clustering memberships and medoids using sk-medoid algorithm.
//' @param x A distance matrix.
//' @param nclust A number of clusters.
//' @param iterate A number of iteration of each procedure.
//' @param seeding A number of seeding replications.
//'
//' @details This is a partitioning algorithm that group objects based on the minumum total distance.
//' @author Weksi Budiaji
//' @seealso kmeanslike
//' @import RcppArmadillo
//' @export
// [[Rcpp::export]]
List skmedoidcpp(NumericMatrix x, int nclust, int iterate = 10, int seeding = 20) {

  //*** cluster result //
  IntegerMatrix R(seeding, nclust);
  NumericVector vecE(seeding);
  int n = x.nrow();
  arma::mat m = as<arma::mat>(x);
  IntegerVector memretain = seq_len(nclust) - 1;

	NumericVector rowmat, sumat(n);//, sumorder;
    for (int i = 0; i < n; i ++) {
      rowmat = x(i,_);
      sumat[i] = sum(rowmat);
    }
	arma::uvec init0 = as<arma::uvec>(sumat);
  	arma::uvec initidx0 = sort_index(init0);
  	IntegerVector idx1rcpp = wrap(initidx0);
  	IntegerVector idxno0 = seq_len(n-1);
  	IntegerVector idx1no0 = idx1rcpp[idxno0];

  	IntegerMatrix newinit(seeding,nclust-1);

	for (int start = 0; start < seeding; start++) {
  		newinit(start,_) = sample(idx1no0, nclust-1, FALSE);
  	}
  	IntegerMatrix firstmedo(seeding, 1);
	firstmedo(_,0) = rep(idx1rcpp[0], seeding);

  	arma::mat armafirst = as<arma::mat>(firstmedo);
  	arma::mat armasecond = as<arma::mat>(newinit);
  	arma::mat armainitial = join_rows(armafirst, armasecond);

  	IntegerMatrix initialrcpp = wrap(armainitial);

  for (int start = 0; start < seeding; start++) {

   IntegerVector newinit = initialrcpp(start,_);
    newinit = newinit.sort();

    //*** find membership given distance matrix //
    arma::uvec index = as<arma::uvec>(newinit);
    arma::mat M = m.cols(index);
    NumericMatrix N = Rcpp::wrap(M);
    IntegerVector member0(n);
    for (int i = 0; i < n; i++){
      NumericVector cm1 = N(i,_);
      member0[i] = which_min(cm1);
    }
  	//IntegerVector memretain = seq_len(nclust) - 1;
  	member0[newinit] = memretain; //tambahannya

    //*** find medoid each iteration //
    IntegerVector z = seq_len(n) - 1;
    arma::uvec idx = as<arma::uvec>(member0);
    IntegerVector medoid1(nclust);
    for (int i = 0; i < nclust; i++) {
      arma::mat M1 = m.submat(find(idx == i), find(idx == i));
      NumericMatrix N1 = Rcpp::wrap(M1);
      int nsub = N1.nrow();
      IntegerVector q = z[member0 == i];
      NumericVector out(nsub), idclass(nsub);
      for (int j = 0; j < nsub; j++) {
        out[j] = sum(N1(j,_));
        idclass[j] = q[j];
      }
      medoid1[i] =  idclass[which_min(out)];
    }

    medoid1 = medoid1.sort(); //tambahannya

  	IntegerVector medoid2(nclust), medoid; //vector length = nclust

    if (sum(medoid1 != newinit)!=0) {
    int a = 0;
    while ( a < iterate) {
      //*** find membership given distance matrix //
      	arma::uvec index2 = as<arma::uvec>(medoid1);
      	arma::mat M2 = m.cols(index2);
     	NumericMatrix N2 = Rcpp::wrap(M2);
      	IntegerVector member1(n);
      	for (int i = 0; i < n; i++){
        	NumericVector cm2 = N2(i,_);
        	member1[i] = which_min(cm2);
      	}
      	//IntegerVector memretain = seq_len(nclust) - 1;
  		member1[medoid1] = memretain; //tambahannya

      //*** find medoid each iteration //
      	arma::uvec idx2 = as<arma::uvec>(member1);
      	for (int i = 0; i < nclust; i++) {
        	arma::mat M3 = m.submat(find(idx2 == i), find(idx2 == i));
        	NumericMatrix N3 = Rcpp::wrap(M3);
        	int nsub1 = N3.nrow();
        	IntegerVector q1 = z[member1 == i];
        	NumericVector out1(nsub1), idclass1(nsub1);
        	for (int j = 0; j < nsub1; j++) {
          		out1[j] = sum(N3(j,_));
          		idclass1[j] = q1[j];
        	}
       	 	medoid2[i] =  idclass1[which_min(out1)];
      	}

      	medoid2 = medoid2.sort(); //tambahannya

      	if (sum(medoid1 != medoid2) == 0) { break;}
      	medoid1 = medoid2;
      	a++;
    }
    medoid = medoid2;

  	} else {
    medoid = medoid1;

  	}

  	R(start,_) = medoid;

  //*** find membership and loss value given distance matrix //
  	arma::uvec index3 = as<arma::uvec>(medoid);
  	arma::mat M4 = m.cols(index3);
  	NumericMatrix N4 = Rcpp::wrap(M4);
  	NumericVector E(n);
  	for (int i = 0; i < n; i++){
    	NumericVector cm3 = N4(i,_);
    	E[i] = min(cm3);
  	}
  	vecE[start] = sum(E);

  }

  int sk = which_min(vecE);
  arma::mat Rm = as<arma::mat>(R);
  arma::mat RM = Rm.rows(sk,sk);
  IntegerMatrix RN = Rcpp::wrap(RM);
  IntegerVector finalmedo = RN(0,_);

  arma::uvec index4 = as<arma::uvec>(finalmedo);
  arma::mat M5 = m.cols(index4);
  NumericMatrix N5 = Rcpp::wrap(M5);
  IntegerVector finalmember(n);
  NumericVector EE(n);
  for (int i = 0; i < n; i++){
    NumericVector cm4 = N5(i,_);
    finalmember[i] = which_min(cm4);
    EE[i] = min(cm4);
  }
  finalmember[finalmedo] = memretain; //tambahannya

  float finalE = sum(EE);

  return List::create(Named("cluster") = finalmember + 1, Named("medoid") = finalmedo + 1,
                      Named("minimum_distance") = finalE);
}


